const getGQL = url =>
(query, variables) => fetch(url, {
    method: 'POST',
    headers: {
        "Content-Type": "application/json",
        ...(localStorage.authToken ? {"Authorization": "Bearer " + localStorage.authToken} : {})
    },
    body: JSON.stringify({query, variables})
}).then(res => res.json())
    .then(data => {
        if (data.data){
            return Object.values(data.data)[0] 
        } 
        else throw new Error(JSON.stringify(data.errors))
    })

export const backendURL = 'http://hipstagram.asmer.fs.a-level.com.ua';

const gql = getGQL(backendURL + '/graphql');





export const actionPromise = (name,promise) =>{
 const actionPending = name => ({name,type:"PROMISE",status:'PENDING'})
 const actionFulfiled = (name,payload) =>({name,type:"PROMISE",status:'FULFILLED',payload})
 const actionRejected = (name,error) => ({name,type:"PROMISE",status:'REJECTED',error})


 return async dispatch => {
    dispatch(actionPending(name))
    try{
        let payload = await promise
        dispatch(actionFulfiled(name,payload))
        return payload
    }
    catch(e){
        dispatch(actionRejected(name,e))
    }
 }
}


export const actionAuthLogin = (tokennn) =>({type:'AUTH_LOGIN',token:tokennn})
export const actionAuthLogout= () => ({type: 'AUTH_LOGOUT'})







export const actionFullRegister = (login,password) => async (dispatch, getState) => {
      await dispatch(
        actionPromise('fullRegister', gql(`mutation register($login:String!,$password:String!){
          UserUpsert(user:{login:$login,password:$password}){
            _id login
          }
        }`,{login: login, password: password}))
      );
      const {
        promise: {fullRegister}
      } = getState();
      if (fullRegister.status === "FULLFILLED") {
          dispatch(actionFullLogin(login,password))
      }
}
          

export const actionFullLogin = (login,password) => async (dispatch,getState) => {
   const tokennn = await dispatch(
    actionPromise('fullLogin', gql(`query login($login:String!,$password:String!){
        login(login:$login,password:$password)
    }`,{login:login,password:password}))
   );
   await dispatch(actionAuthLogin(tokennn));
   const {
     auth: {payload}
   } = getState();
   await dispatch(actionAboutMe(payload.sub.id));
}


export const uploadFile = file => {
  const fd = new FormData();
  fd.append('photo', file);
return  fetch(backendURL+'/upload', {
    method: "POST",
    headers: localStorage.authToken ? {Authorization: 'Bearer ' + localStorage.authToken} : {},
    body: fd
}).then(res => res.json());

}



export const actionUploadFile = file => 
  actionPromise('fileUpload', uploadFile(file));

export const actionUploadFiles = files =>
   actionPromise('filesUpload',Promise.all(files.map(file => uploadFile(file))))

export const actionAvatar = (userrId, fileeId) => 
  actionPromise("userAvatar", gql(`mutation setAvatar($userId: String, $fileId: ID){
    UserUpsert(user:{_id: $userId, avatar: {_id: $fileId}}){
        _id, avatar{
            _id
        }
    }
}`, {userId: userrId,
      fileId: fileeId}))

export const actionAboutMe = (_id) => 
        actionPromise("aboutMe", gql(`query AboutMe ($Id: String){
            UserFindOne(query:$Id){
              _id,nick, avatar{
                url
              },
              following{
                _id
              }
            }
          }`,{
              Id: JSON.stringify([{_id}])
          }))

export const actionAboutUser = (_id) =>
        actionPromise("aboutUser", gql(`query AboutUser ($Id: String){
          UserFindOne(query:$Id){
            _id,nick, avatar{
              url
            },
            following{
              _id
            }
          }
        }`,{
            Id: JSON.stringify([{_id}])
        }))          

export const actionSetAvatar = file => 
          async(dispatch, getState) => {
             await dispatch(actionUploadFile(file))
             const {auth:{payload:{sub:{id}}},promise:{fileUpload:{payload:{_id}}}} = getState() 
            await dispatch(actionAvatar(id,_id))
            await dispatch(actionAboutMe(id))
          }
export const actionPostUpsert = post =>
          actionPromise("postUpsert", gql(`mutation PostUpsert ($post: PostInput){
            PostUpsert(post:$post){
              _id
            }
          }`, {post:{...post,images:post.images.map(({_id})=>({_id}))}}))



export const actionFindUser = nick =>
          actionPromise("findUsers", gql(`query FindUsers($query: String){
            UserFind(query:$query){
              _id,nick,avatar{
                url
              }
            }
          }`, {query: JSON.stringify([
            {
              $or: [{nick:`/${nick}/`}]
            },
            {
              sort:[{nick:1}]
            }
          ])}))
          
export const actionUserPost = _id =>
        actionPromise("userPost", gql(`query PostFind($filter: String){
          PostFind(query:$filter){
                    owner{_id nick avatar{url}}
                    _id likesCount title images{_id url}
                      comments{
                        _id text post{_id} likesCount owner{_id nick avatar{url}}
                        
                      }
                    likes{
                        owner{				
                            login avatar {url}
                          }
                    }
              }
            }`,{
              filter: JSON.stringify([
                {
                  ___owner:{$in:[_id]}
                }
              ])
            }))      
            
            
            /* {query: JSON.stringify([
              {___owner: {$in :[`${_id}`]}}
            ])} */

